# -*- coding: utf-8 -*-
from setuphelpers import *
import platform

uninstallkey = []

# Defining variables
bin_name_sub = 'StarLeaf-%s-Windows.exe'
silent_args = '/S'
app_uninstallkey = ''
app_dir = makepath(programfiles, 'StarLeaf')


def install():
    # Initializing variables
    package_version = control.version.split('-',1)[0]
    bin_name = bin_name_sub % package_version
    bin_path = makepath(app_dir,bin_name)
    processes_to_kill = control.impacted_process.split(',')

    # Installing the package
    if not isdir(app_dir):
        print("Creating: %s directory" % app_dir)
        mkdirs(app_dir)
    if isfile(bin_path):
        remove_file(bin_path)
    print("Copying: %s to: %s" % (bin_name,app_dir))
    filecopyto(bin_name,app_dir)


def uninstall():
    # Initializing variables
    processes_to_kill = control.impacted_process.split(',')

    # Uninstalling the package
    if processes_to_kill:
        killalltasks(processes_to_kill)
    if isdir(app_dir):
        remove_tree(app_dir)


def session_setup():
    # Initializing variables
    package_version = control.version.split('-')[0]
    bin_name = bin_name_sub % package_version
    bin_path = makepath(app_dir,bin_name)
    app_name = control.name
    processes_to_kill = control.impacted_process.split(',')
    #"C:\Users\username\AppData\Local\StarLeaf\StarLeaf\StarLeaf.exe"
    user_app_dir = makepath(user_local_appdata, 'StarLeaf', 'StarLeaf')
    user_app_bin = makepath(user_app_dir, 'StarLeaf.exe')
    user_app_uninst_path = makepath(user_app_dir, 'Misc', 'Uninstall.exe')
    user_app_bin_version = get_version_from_binary(user_app_bin)
    #user_app_reg_version = registry_readstring(HKEY_CURRENT_USER,r'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\StarLeaf','DisplayVersion')

    # Installing the package in user environment
    if user_app_bin_version == '' or Version(package_version) > Version(user_app_bin_version):
        # Uninstall may be required a day
        killalltasks(processes_to_kill)
        print("Removing: %s (%s) from user environment" % (app_name, user_app_bin_version))
        run('"%s" %s' % (user_app_uninst_path, silent_args))
        print("Installing: %s in user environment" % bin_name)
        """ killalltasks(processes_to_kill)
        print('"%s" %s' % (bin_path, silent_args))
        run('"%s" %s' % (bin_path, silent_args)) """
        install_exe_if_needed(bin_path,
            silentflags=silent_args,
            key=app_uninstallkey,
            killbefore=processes_to_kill,
            min_version=package_version)
    else:
        print("OK: %s is already installed, or %s is a newer version than %s." % (app_name, user_app_bin_version, package_version))


def update_package():
    # Declaring local variables
    result = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    url_dl = 'https://dl.starleaf.com/app/StarLeaf-Installer-Windows.exe'
    bin_temp = 'StarLeaf-Installer-Windows.exe'
    version = '1.0'
    latest_bin = bin_temp

    print("Download url is: %s" % url_dl)

    # Downloading latest binaries
    if not isfile(latest_bin):
        print('Downloading: %s' % latest_bin)
        wget(url_dl, latest_bin, proxies=proxies)

        # Checking version from file
        version_from_file = get_version_from_binary(latest_bin)
        if Version(version) != Version(version_from_file) and version_from_file != '':
            print("Changing version to the version number of the binary (from: %s to: %s)" % (version, version_from_file))
            os.rename(latest_bin, bin_name_sub % version_from_file)
            version = version_from_file

    # Changing version of the package
    if Version(version) > Version(control.get_software_version()):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        result = True
    control.version = '%s-%s' % (Version(version), control.version.split('-', 1)[-1])
    #control.set_software_version(Version(version))
    control.save_control_to_wapt()

    # Deleting outdated binaries
    remove_outdated_binaries(version)

    # Validating or not update-package-sources
    return result





def get_proxies():
    r"""Return system proxy with the urllib python library

    >>> get_proxies()
    {'http': 'http://srvproxy.ad.domain.lan:8080',
    'https': 'http://srvproxy.ad.domain.lan:8080'}

    """
    if platform.python_version_tuple()[0] == '3':
        from urllib.request import getproxies
    else:
        from urllib import getproxies
    return getproxies()


def get_proxies_from_wapt_console():
    r"""Return proxy information from the current user WAPT console

    >>> get_proxies_from_wapt_console()
    {'http': 'http://srvproxy.ad.domain.lan:8080',
    'https': 'http://srvproxy.ad.domain.lan:8080'}

    """
    proxies = {}
    if platform.system() == 'Windows':
        waptconsole_ini_path = makepath(user_local_appdata(), 'waptconsole', 'waptconsole.ini')
    else:
        waptconsole_ini_path = makepath(user_home_directory(), '.config', 'waptconsole', 'waptconsole.ini')
    if isfile(waptconsole_ini_path):
        proxy_wapt = inifile_readstring(waptconsole_ini_path, 'global', 'http_proxy')
        if proxy_wapt:
            proxies = {'http': proxy_wapt, 'https': proxy_wapt}
    return proxies


def get_version_from_binary(filename, property_name='ProductVersion'):
    r""" Get installer version from file informations, for now, only exe and msi files are compatibles

    Args:
        filename (str): path to the file
        property_name (str): selected property

    Returns:
        str: version number

     """
    if filename.endswith('.msi'):
        return get_msi_properties(filename)[property_name]
    else:
        return get_file_properties(filename)[property_name]


def remove_outdated_binaries(version, filename_contains=None, list_extensions=['exe','msi','deb','rpm','dmg','pkg']):
    r"""Remove files based on the version contained in his filename

    Args:
        version (str): version number of keeped files
        filename_contains (str or list of str): Part of the filename that must be contained (useful for distinguishing architecture and os)
        list_extensions (str or list of str): file extensions of verified files

    Returns:
        None

    .. versionadded:: 2.0

    """
    if type(list_extensions) != list:
        list_extensions = [list_extensions]
    if filename_contains:
        if type(filename_contains) != list:
            filename_contains = [filename_contains]
    list_extensions = ['.' + ext for ext in list_extensions if ext[0] != '.']
    for file_ext in list_extensions:
        for bin_in_dir in glob.glob('*%s' % file_ext):
            if not version in bin_in_dir:
                remove_file(bin_in_dir)
            if filename_contains:
                for filename_contain in filename_contains:
                    if not filename_contain in bin_in_dir:
                        remove_file(bin_in_dir)

